==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/client/inc/obj_suggest.js <==
/*****************************************************
 * Input form object with suggestions
 * This object provides opportunity to suggest users input by searching in given array
 * or by programming _query method and make queries to server for every input change.
 *
 * CSS
 *
 *
 * Parametrs:
 *          _min = int      // minimal no. of letters for suggestion
 *          _limit = int    // max no of suggestions
 *
 * Methods:
 *          _listen()
 *
 * Dynamic querying mode:
 *         obj._dynamic = true // choose query mode
 *         obj._listen('sDataSet')
 *         obj._query = function(){
 *                        this.__eIN.value // input value
 *                        this.__bTimer    // 500ms delay between querying is in progres
 *                        this.__bQuerying // querying is in progres
 *
 *                        <query>
 *
 *                        return Array;    // 2D array [string,string...]
 *                      }
 *
 * Object behaviour:
 *        onfocus   - check for suggestions
 *        onblur    - close suggestions
 *        onkeydown - enter - set value from suggested, close suggestions
 *                    right - set value from suggested
 *                    up/dn - change position in suggetstions
 *                            top pos. + up - close suggestions
 *                            top pos. + dn - check suggest
 *        onkeyup   - check for suggestions
 *
 *
 *
 * 2Do:
 * - doresit query timeout
 *****************************************************/

_me = obj_suggest.prototype;
function obj_suggest(){};

/**
 * @brief: CONSTRUCTOR, create input with suggestion form object to __eIN variable
 * @date : 20.4.2006 17:24:21
 **/
_me.__constructor = function(){
	var me = this;

	this.__hideTime;
	this.__showTime;
	this.__sLastSuggest = '';
	this.__sQuery_data = '';

	/* PUBLIC properties */
	this._min = 3;
	this._limit = 15;

	this.__cleanDS = true; // object relieve its DS

	//this._dynamic = false;

	/* listener for suggestion */
	this._listener_data;
	this._listenerPath_data;

    this._obeyEvent('onfocus',[this,'__onfocus']);
    this._obeyEvent('onblur',[this,'__onblur']);
	//this._obeyEvent('onkeydown',[this,'__onkeydown']);
	this._obeyEvent('onkeyup',[this,'__onkeyup']);
	this._obeyEvent('onclick',[this,'__onclick']);

	// add destructirs
    this._add_destructor('__destructor');
    
    //Inherite
	this._listen_data('obj_suggest_'+this._pathName);
};

	_me.__onfocus = function(){
		if (this.__hideTime){
			window.clearTimeout(this.__hideTime);
			this.__hideTime = undefined;
		}
			

		this.__showTime = window.setTimeout("try{"+this._pathName+".__suggest();}catch(e){}",500);
	};

	_me.__onblur = function(){
		if (this.__showTime){
			window.clearTimeout(this.__showTime);
			this.__showTime = undefined;
		}

		this.__hideTime = window.setTimeout("try{"+this._pathName+".__hide();}catch(e){}",10);
	};

	_me._onkeydown = function(e){
		if (this.__showTime){
		    window.clearTimeout(this.__showTime);
		    this.__showTime = undefined;
		}

		switch(e.keyCode){
        case 41:  // Right
            break;
		case 38:  // Up
			if (this.suggest){
				this.__up();
				return false;
			}
			break;
		case 39:  // Right
			break;
		case 40:  // Down
			if (this.suggest){
				this.__dn();
				return false;
			}
			break;
		case 13:
			if (this.__eActive){
	        	this._qvalue(this.__eActive.innerHTML.unescapeHTML());
	        	this.__hide();
	        }
		}
	};

	_me.__onkeyup = function(e){
		if(!e.keyCode || inArray([9,40,38,13,27],e.keyCode)>-1) return;
		this.__showTime = window.setTimeout("try{"+this._pathName+".__suggest();}catch(e){}",500);
	};
	
	_me.__onclick = function(e){
		if (!e.contextmenu){
			if (this.__hideTime){
				window.clearTimeout(this.__hideTime);
				this.__hideTime = undefined;
			}

			this.__showTime = window.setTimeout("try{"+this._pathName+".__suggest();}catch(e){}",500);
		}
	};
	
	_me._onclose = function(e){

		if (this.suggest){
			e.preventDefault();
			e.cancelBubble = true;

			this.__hide();
			return false;
		}

		return true;
	};
/**
 * @brief: Show UL-LI suggestion element and align it with input
 * @date : 20.4.2006 17:27:33
 **/
_me.__show = function(){

	if (this.__hideTime){
		window.clearTimeout(this.__hideTime);
		this.__hideTime = undefined;
	}

	if(!this.__showTime) return;
	
	if (!this.__sQuery_data.length){
		this.__hide();
		return;
	}

	if(!this.suggest){
		this.suggest = gui._create('suggest','obj_block','','gui_suggest_block');
		this.suggest._position('absolute');
		
		var me = this;
        this.suggest._main.onmousedown = function(e){
        	var e = e || window.event;
        	var elm = e.target || e.srcElement;
        	if (elm.tagName == 'A')
        	    me._qvalue(elm.innerHTML.unescapeHTML());
		};
		
        this.suggest._main.onmousemove = function(e){
        	var e = e || window.event;
        	var elm = e.target || e.srcElement;
        	if (elm.tagName == 'A'){
				removecss(me.__eActive,'active');
				me.__eActive  = elm;
        	    addcss(elm,'active');
			}
		};

	}

	var oPos = getSize(this._main);
	this.suggest._main.innerHTML = this.__sQuery_data;
	this.suggest._place(oPos.x,oPos.y+oPos.h,oPos.w);
	this.suggest._focus();
	
	if (!this.__eActive){
		this.__eActive = this.suggest._main.getElementsByTagName('A');
		if (this.__eActive.length){
            this.__eActive = this.__eActive[0];
			addcss(this.__eActive,'active');
		}
	}
};

/**
 * @brief: Hide UL-LI suggestion element and clear "active row"
 * @date : 21.4.2006 16:19:08
 **/
_me.__hide = function(){
	if (this.__hideTime){
		window.clearTimeout(this.__hideTime);
		this.__hideTime = undefined;
	}

	this.__eActive = null;
	this.__eActive = undefined;

	if (this.suggest){
		this.suggest._destruct();
		this.suggest = null;
		this.suggest = undefined;
	}
};

_me.__up = function(){
	var elm = this.__eActive.previousSibling;
	if (elm && elm.tagName=='A'){
        removecss(this.__eActive,'active');
        this.__eActive = elm;
		addcss(elm,'active');
	}
};
_me.__dn = function(){
	var elm = this.__eActive.nextSibling;
	if (elm && elm.tagName=='A'){
        removecss(this.__eActive,'active');
        this.__eActive = elm;
		addcss(elm,'active');
	}
};

/**
 * @brief: Object destructor
 * @date : 21.4.2006 16:36:17
 **/
_me.__destructor = function(){
	this.__hide();
	this.__eIN = null;

	if (this.__cleanDS)
		dataSet.remove(this._listener_data,this._listenerPath_data);
};

_me.__suggest = function(){
    if (!this.__showTime) return;

	var v = this._qdata(this.__eIN.value);
	if (!v || v.length<this._min){
        this.__sLastSuggest = '';
		this.__hide();
		return;
	}

	if (this.__sLastSuggest != v){
		this._query(v);
		this.__sLastSuggest = v;
	}
	else
	if (!this.suggest)
		this._parse();
};

_me.__update = function(sDataSet,sDataPath){
	if (!sDataSet) return;

	// update suggest
	if (this._listener_data == sDataSet)
		this._parse(dataSet.get(this._listener_data,this._listenerPath_data));
	// update value
	else
	if (this._listener == sDataSet)
		this._value(dataSet.get(this._listener,this._listenerPath));
};

_me._listen_data = function(sDataSet,aDataPath){
	this._listener_data = sDataSet;
	if (typeof aDataPath == 'object') this._listenerPath_data = aDataPath;
	dataSet.obey(this,'_listener_data',sDataSet);
};

    /**** Inherite Part ****
	_me._qvalue = function(v){
        this._value(v);
		this._setRange(v);
	};

	_me._qdata = function(v){
		return v;
	};



	_me._query = function(v){
		if (!this._listener_data) return;
		// add query here
	};

	_me._parse = function(aValues){

		if (!aValues){
            this.__show();
			return;
		}

		this.__eActive = null;
		this.__sQuery_data = '';

		if (!count(aValues)) return;

		for(var i in aValues){
			this.__sQuery_data +='<a href="javascript: void(0);">'+ aValues[i].escapeHTML() +"</a>";
		}
		
		this.__show();
	};
	*/

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/client/inc/obj_suggest.js <==
/*****************************************************
 * Input form object with suggestions
 * This object provides opportunity to suggest users input by searching in given array
 * or by programming _query method and make queries to server for every input change.
 *
 * CSS
 *
 *
 * Parametrs:
 *          _min = int      // minimal no. of letters for suggestion
 *          _limit = int    // max no of suggestions
 *
 * Methods:
 *          _listen()
 *
 * Dynamic querying mode:
 *         obj._dynamic = true // choose query mode
 *         obj._listen('sDataSet')
 *         obj._query = function(){
 *                        this.__eIN.value // input value
 *                        this.__bTimer    // 500ms delay between querying is in progres
 *                        this.__bQuerying // querying is in progres
 *
 *                        <query>
 *
 *                        return Array;    // 2D array [string,string...]
 *                      }
 *
 * Object behaviour:
 *        onfocus   - check for suggestions
 *        onblur    - close suggestions
 *        onkeydown - enter - set value from suggested, close suggestions
 *                    right - set value from suggested
 *                    up/dn - change position in suggetstions
 *                            top pos. + up - close suggestions
 *                            top pos. + dn - check suggest
 *        onkeyup   - check for suggestions
 *
 *
 *
 * 2Do:
 * - doresit query timeout
 *****************************************************/

_me = obj_suggest.prototype;
function obj_suggest(){};

/**
 * @brief: CONSTRUCTOR, create input with suggestion form object to __eIN variable
 * @date : 20.4.2006 17:24:21
 **/
_me.__constructor = function(){
	var me = this;

	this.__hideTime;
	this.__showTime;
	this.__sLastSuggest = '';
	this.__sQuery_data = '';

	/* PUBLIC properties */
	this._min = 3;
	this._limit = 15;

	this.__cleanDS = true; // object relieve its DS

	//this._dynamic = false;

	/* listener for suggestion */
	this._listener_data;
	this._listenerPath_data;

    this._obeyEvent('onfocus',[this,'__onfocus']);
    this._obeyEvent('onblur',[this,'__onblur']);
	//this._obeyEvent('onkeydown',[this,'__onkeydown']);
	this._obeyEvent('onkeyup',[this,'__onkeyup']);
	this._obeyEvent('onclick',[this,'__onclick']);

	// add destructirs
    this._add_destructor('__destructor');
    
    //Inherite
	this._listen_data('obj_suggest_'+this._pathName);
};

	_me.__onfocus = function(){
		if (this.__hideTime){
			window.clearTimeout(this.__hideTime);
			this.__hideTime = null;
		}
			
		if (this.__showTime)
			window.clearTimeout(this.__showTime);
		this.__showTime = window.setTimeout("try{"+this._pathName+".__suggest();}catch(e){}",500);
	};

	_me.__onblur = function(){
		if (this.__showTime){
			window.clearTimeout(this.__showTime);
			this.__showTime = null;
		}

		if (this.__hideTime)
			window.clearTimeout(this.__hideTime);
		this.__hideTime = window.setTimeout("try{"+this._pathName+".__hide();}catch(e){}",10);
	};

	_me._onkeydown = function(e){
		if (this.__showTime){
		    window.clearTimeout(this.__showTime);
		    this.__showTime = null;
		}

		switch(e.keyCode){
        case 41:  // Right
            break;
		case 38:  // Up
			if (this.suggest){
				this.__up();
				return false;
			}
			break;
		case 39:  // Right
			break;
		case 40:  // Down
			if (this.suggest){
				this.__dn();
				return false;
			}
			break;
		case 13:
			if (this.__eActive){
	        	this._qvalue(this.__eActive.innerHTML.unescapeHTML());
	        	this.__hide();
	        }
		}
	};

	_me.__onkeyup = function(e){
		if(!e.keyCode || inArray([9,40,38,13,27],e.keyCode)>-1) return;

		if (this.__showTime)
			window.clearTimeout(this.__showTime);
		this.__showTime = window.setTimeout("try{"+this._pathName+".__suggest();}catch(e){}",500);
	};
	
	_me.__onclick = function(e){
		if (!e.contextmenu){
			if (this.__hideTime){
				window.clearTimeout(this.__hideTime);
				this.__hideTime = null;
			}

			if (this.__showTime)
				window.clearTimeout(this.__showTime);
			this.__showTime = window.setTimeout("try{"+this._pathName+".__suggest();}catch(e){}",500);
		}
	};
	
	_me._onclose = function(e){

		if (this.suggest){
			e.preventDefault();
			e.cancelBubble = true;

			this.__hide();
			return false;
		}

		return true;
	};
/**
 * @brief: Show UL-LI suggestion element and align it with input
 * @date : 20.4.2006 17:27:33
 **/
_me.__show = function(){

	if (this._destructed) return;

	if (this.__hideTime){
		window.clearTimeout(this.__hideTime);
		this.__hideTime = null;
	}

	if (!this.__showTime) return;

	if (!this.__sQuery_data.length){
		this.__hide();
		return;
	}

	if (!this.suggest){
        if (this._destructed) return;
		this.suggest = gui._create('suggest','obj_block','','gui_suggest_block');
		this.suggest._position('absolute');

		var me = this;
		this.suggest._main.onmousedown = function(e){
		var e = e || window.event;
		var elm = e.target || e.srcElement;
		if (elm.tagName == 'A')
			me._qvalue(elm.innerHTML.unescapeHTML());
		};

		this.suggest._main.onmousemove = function(e){
			var e = e || window.event;
			var elm = e.target || e.srcElement;
			if (elm.tagName == 'A'){
				removecss(me.__eActive,'active');
				me.__eActive = elm;
				addcss(elm,'active');
			}
		};

	}

	var oPos = getSize(this._main);
	this.suggest._main.innerHTML = this.__sQuery_data;
	this.suggest._place(oPos.x,oPos.y+oPos.h,oPos.w);
	this.suggest._focus();

	if (!this.__eActive){
		this.__eActive = this.suggest._main.getElementsByTagName('A');
		if (this.__eActive.length){
			this.__eActive = this.__eActive[0];
			addcss(this.__eActive,'active');
		}
	}
};

/**
 * @brief: Hide UL-LI suggestion element and clear "active row"
 * @date : 21.4.2006 16:19:08
 **/
_me.__hide = function(){
	if (this.__hideTime){
		window.clearTimeout(this.__hideTime);
		this.__hideTime = '';
	}

	this.__eActive = null;

	if (this.suggest){
		this.suggest._destruct();
		this.suggest = null;
	}
};

_me.__up = function(){
	var elm = this.__eActive.previousSibling;
	if (elm && elm.tagName=='A'){
		removecss(this.__eActive,'active');
		this.__eActive = elm;
		addcss(elm,'active');
	}
};
_me.__dn = function(){
	var elm = this.__eActive.nextSibling;
	if (elm && elm.tagName=='A'){
		removecss(this.__eActive,'active');
		this.__eActive = elm;
		addcss(elm,'active');
	}
};

/**
 * @brief: Object destructor
 * @date : 21.4.2006 16:36:17
 **/
_me.__destructor = function(){

	if (this.__showTime){
		window.clearTimeout(this.__showTime);
		this.__showTime = null;
	}

	this.__hide();
	
	this.__eIN = null;
	if (this.__cleanDS)
		dataSet.remove(this._listener_data,this._listenerPath_data);
};

_me.__suggest = function(){
	if (!this.__showTime) return;

	this.__input_value = this.__eIN.value;

	var v = this._qdata(this.__input_value);
	if (!v || v.length<this._min){
		this.__sLastSuggest = '';
		this.__hide();
		return;
	}

	if (this.__sLastSuggest != v){
		this._query(v);
		this.__sLastSuggest = v;
	}
	else
	if (!this.suggest)
		this._parse();
};

_me.__update = function(sDataSet,sDataPath){
	if (!sDataSet) return;

	// update suggest
	if (this._listener_data == sDataSet)
		this._parse(dataSet.get(this._listener_data,this._listenerPath_data));
	// update value
	else
	if (this._listener == sDataSet)
		this._value(dataSet.get(this._listener,this._listenerPath));
};

_me._listen_data = function(sDataSet,aDataPath){
	this._listener_data = sDataSet;
	if (typeof aDataPath == 'object') this._listenerPath_data = aDataPath;
	dataSet.obey(this,'_listener_data',sDataSet);
};

    /**** Inherite Part ****
	_me._qvalue = function(v){
		this._value(v);
		this._setRange(v);
	};

	_me._qdata = function(v){
		return v;
	};



	_me._query = function(v){
		if (!this._listener_data) return;
		// add query here
	};

	_me._parse = function(aValues){

		if (!aValues){
            this.__show();
			return;
		}

		this.__eActive = null;
		this.__sQuery_data = '';

		if (!count(aValues)) return;

		for(var i in aValues){
			this.__sQuery_data +='<a href="javascript: void(0);">'+ aValues[i].escapeHTML() +"</a>";
		}
		
		this.__show();
	};
	*/
